function [X, info] = ml_sylv_sgn(A, B, C, opts)
%ML_SYLV_SGN Standard Sylvester equation solver.
%
% DESCRIPTION:
%   Computes the solution of the Sylvester equation
%
%       A*X + X*B = C,                                                  (1)
%
%   using the sign function iteration. It is assumed that the eigenvalues
%   of A and B lie in the open left half-plane.
%
% SIGNATURE:
%   [X, info] = ML_SYLV_SGN(A, B, C)
%   [X, info] = ML_SYLV_SGN(A, B, C, opts)
%
% INPUT:
%   A    - matrix with dimensions n x n from (1)
%   B    - matrix with dimensions m x m from (1)
%   C    - matrix with dimensions n x m from (1)
%   opts - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | AbsTol          | nonnegative scalar, tolerance for the absolute    |
%   |                 | error in the last iteration step                  |
%   |                 | (default 0)                                       |
%   |-----------------|---------------------------------------------------|
%   | Info            | {0, 1}, used to disable/enable display of verbose |
%   |                 | status information during iteration steps         |
%   |                 | (default 0)                                       |
%   |-----------------|---------------------------------------------------|
%   | MaxIter         | positive integer, maximum number of iteration     |
%   |                 | steps                                             |
%   |                 | (default 100)                                     |
%   |-----------------|---------------------------------------------------|
%   | RelTol          | nonnegative scalar, tolerance for the relative    |
%   |                 | error in the last iteration step                  |
%   |                 | (default 1.0e+01*n*eps)                           |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   X    - solution of the Sylvester equation (1)
%   info - structure, containing the following information:
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | AbsErr          | vector, containing the absolute error of the      |
%   |                 | iteration matrix in each iteration step           |
%   |-----------------|---------------------------------------------------|
%   | IterationSteps  | number of performed iteration steps               |
%   |-----------------|---------------------------------------------------|
%   | RelErr          | vector, containing the relative error of the      |
%   |                 | iteration matrix in each iteration step           |
%   |-----------------|---------------------------------------------------|
%
%
% REFERENCE:
%   P. BENNER. Factorized Solution of Sylvester Equations with Applications
%   in Control. In "Proceedings of the 16th International Symposium on:
%   Mathematical Theory of Networks and Systems, MTNS 2004, Leuven,
%   Belgium, July 5-9, 2004", pages 1-10, Leuven, Belgium, 2004.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(3, 4);

if (nargin < 4) || isempty(opts)
    opts = struct();
end

% Check input matrices.
n = size(A, 1);
m = size(B, 1);

assert(isequal(size(A), [n n]), ...
    'MORLAB:data', ...
    'The matrix A has to be square!');

assert(isequal(size(B), [m m]), ...
    'MORLAB:data', ...
    'The matrix B has to be square!');

assert(isequal(size(C), [n m]), ...
    'MORLAB:data', ...
    'The matrix C must have the dimensions d x d!', ...
    n, m);

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'AbsTol')
    ml_assert_nonnegscalar(opts.AbsTol, 'opts.AbsTol');
else
    opts.AbsTol = 0;
end

if ml_field_set_to_value(opts, 'Info')
    ml_assert_boolean(opts.Info, 'opts.Info');
else
    opts.Info = false;
end

if ml_field_set_to_value(opts, 'MaxIter')
    ml_assert_posinteger(opts.MaxIter, 'opts.MaxIter');
else
    opts.MaxIter = 100;
end

if ml_field_set_to_value(opts, 'RelTol')
    ml_assert_nonnegscalar(opts.RelTol, 'opts.RelTol');
else
    opts.RelTol = 1.0e+01 * n * eps;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIALIZATION.                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

niter  = 0;
X      = C;
Aeye   = speye(n);
Beye   = speye(m);

[abserr, relerr] = deal(zeros(1, opts.MaxIter));

normA = norm(A, 'fro');
normB = norm(B, 'fro');

abserr(1) = max([norm(A + Aeye, 'fro'), norm(B + Beye, 'fro')]);
relerr(1) = max([norm(A + Aeye, 'fro') / normA, ...
    norm(B + Beye, 'fro') / normB]);

% Method is converged if absolute or relative errors are small enough.
converged = (abserr(1) <= opts.AbsTol) || (relerr(1) <= opts.RelTol);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIGN FUNCTION ITERATION.                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

while (niter < opts.MaxIter) && ~converged
    Ainv = A \ Aeye;
    Binv = B \ Beye;
    W    = Ainv * X * Binv;
    
    % Scaling factor for convergence acceleration.
    if (niter >= 0) && (abserr(1) > 1.0e-02)
        e = sqrt(norm(A, 'fro')^2 + norm(X, 'fro')^2 + norm(B, 'fro')^2 );
        d = sqrt(norm(Ainv, 'fro')^2 + norm(W, 'fro')^2 + ...
            norm(Binv, 'fro')^2);
        c = sqrt(e / d);
    else
        c = 1.0;
    end
    c1 = 1.0 / (2.0 * c);
    c2 = 0.5 * c;
    
    % Update of iteration matrices.
    A = c1 * A + c2 * Ainv;
    B = c1 * B + c2 * Binv;
    X = c1 * X + c2 * W;
    
    niter         = niter + 1;
    Aerr          = norm(A + Aeye, 'fro');
    Berr          = norm(B + Beye, 'fro');
    abserr(niter) = max([Aerr, Berr]);
    relerr(niter) = max([Aerr / normA, Berr / normB]);
    
    % Information about current iteration step.
    if opts.Info
        fprintf(1, ['SYLV_SGN step: %4d absolute error: %e' ...
            ' relative error: %e \n'], ...
            niter, abserr(niter), relerr(niter));
    end
    
    % Method is converged if absolute or relative errors are small enough.
    converged = (abserr(niter) <= opts.AbsTol) || ...
        (relerr(niter) <= opts.RelTol);
end

X = -0.5 * X;

% Warning if iteration not converged.
if (niter == opts.MaxIter) && ~converged
    warning('MORLAB:eqn_solver', ...
        ['No convergence in %d iteration steps!\n' ...
        'Abs. tolerance: %e | Abs. error: %e\n' ...
        'Rel. tolerance: %e | Rel. error: %e\n' ...
        'Try to increase the tolerances or number of ' ...
        'iteration steps!'], ...
        niter, opts.AbsTol, min(abserr(niter)), ...
        opts.RelTol, min(relerr(niter)));
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN INFORMATION.                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign information about iteration.
info = struct(...
    'AbsErr'        , abserr(1:niter), ...
    'IterationSteps', niter, ...
    'RelErr'        , relerr(1:niter));
