function [varargout] = ml_hna(varargin)
%ML_HNA Hankel-norm approximation for standard systems.
%
% DESCRIPTION:
%   This function computes the Hankel-norm approximation of a standard
%   system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, first a balanced realization is computed by using the
%   balanced truncation square-root method with an appropriate tolerance
%   for the minimal realization of the given system. Then the system is
%   transformed using the formulas given in the reference below. As
%   result, a reduced-order system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G and and the
%   r-th order transfer function Gr it holds
%
%       ||G - Gr||_{H}       = Hsv(r+1),
%       ||G - Gr||_{\infty} <= 2 * (Hsv(r+1) + ... + Hsv(n)),
%
%   with Hsv, a vector containing the Hankel singular values of the system.
%
%   Note: For unstable systems, first an additive decomposition into the
%         stable and anti-stable parts is performed and then only the
%         stable part will be reduced. That does not change the error
%         formulas.
%
% 1) If the standard system is given by matrices:
% SIGNATURE:
%   [Ar, Br, Cr, Dr, info] = ML_BT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_BT(A, B, C, D, opts)
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   opts - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | hankelsignmopts | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition after the transformation of an      |
%   |                 | all-pass system                                   |
%   |                 |   AbsTol    - tolerance for the absolute change   |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative change   |
%   |                 |               (default 1.0e+02*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | hankelsylvopts  | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition after the transformation of an      |
%   |                 | all-pass system                                   |
%   |                 |   AbsTol    - tolerance for the absolute error    |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative error    |
%   |                 |               (default 1.0e+01*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | lyapdlopts      | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time Lyapunov   |
%   |                 | equations, with                                   |
%   |                 |   AbsTol  - tolerance for the absolute error      |
%   |                 |             (default 0)                           |
%   |                 |   CompTol - tolerance for column and row          |
%   |                 |             compression                           |
%   |                 |             (default log(n)*eps)                  |
%   |                 |   Info    - disable/enable display of verbose     |
%   |                 |             status information during iteration   |
%   |                 |             (default 0)                           |
%   |                 |   MaxIter - maximum number of iteration steps     |
%   |                 |             (default 100)                         |
%   |                 |   RelTol  - tolerance for the relative error      |
%   |                 |             (default 1.0e+01*n*eps)               |
%   |-----------------|---------------------------------------------------|
%   | MinRelTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest characteristic value to determine a       |
%   |                 | minimal realization                               |
%   |                 | (default log(length(Hsv))*eps)                    |
%   |-----------------|---------------------------------------------------|
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   |-----------------|---------------------------------------------------|
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   |-----------------|---------------------------------------------------|
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts                                             |
%   |                 |   AbsTol    - tolerance for the absolute change   |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative change   |
%   |                 |               (default 1.0e+02*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | stabsylvopts    | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts                                             |
%   |                 |   AbsTol    - tolerance for the absolute error    |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative error    |
%   |                 |               (default 1.0e+01*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   |-----------------|---------------------------------------------------|
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   info - structure, containing the following information:
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   |                 | reduced-order model in H-infinity norm            |
%   |-----------------|---------------------------------------------------|
%   | Hsv             | a vector, containing the computed Hankel singular |
%   |                 | values                                            |
%   |-----------------|---------------------------------------------------|
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | stable and anti-stable parts, with                |
%   |                 |   infoSTABSIGNM - structure with information      |
%   |                 |                   about the sign function         |
%   |                 |                   iteration used for stabilization|
%   |                 |   infoSTABSYLV  - structure with information      |
%   |                 |                   about the Sylvester equation    |
%   |                 |                   solver used for stabilization   |
%   |                 |   Ns            - number of stable eigenvalues    |
%   |                 |   Nu            - number of anti-stable           |
%   |                 |                   eigenvalues                     |
%   |-----------------|---------------------------------------------------|
%   | infoHASIGNM     | structure, containing information about the       |
%   |                 | matrix sign function method, with                 |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoHASYLV      | structure, containing information about the       |
%   |                 | Sylvester equation solver, with                   |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoLYAPDL      | structure, containing information about the       |
%   |                 | continuous-time dual Lyapunov equations solver,   |
%   |                 | with                                              |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | Ns              | Dimension of the stable part of the reduced-order |
%   |                 | model                                             |
%   |-----------------|---------------------------------------------------|
%   | Nu              | Dimension of the anti-stable part of the reduced- |
%   |                 | order model                                       |
%   |-----------------|---------------------------------------------------|
%   | Sigma           | Chosen Hankel singular value, exact approximation |
%   |                 | error in the Hankel-norm                          |
%   |-----------------|---------------------------------------------------|
%
% 2) If the standard system is given in a struct:
% SIGNATURE:
%   [rom, info] = ML_HNA(sys)
%   [rom, info] = ML_HNA(sys, opts)
%
% INPUT:
%   sys  - structure, containing the standard system's matrices
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (1) with dimensions n x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (2) with dimensions p x n             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (2) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   opts - structure, containing optional parameters (see above 1))
%
% OUTPUT:
%   rom  - structure, with the following entries
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (3) with dimensions r x r             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (3) with dimensions r x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (4) with dimensions p x r             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (4) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   info - structure, containing information (see above 1))
%
% 3) If the standard system is given as state-space object:
% SIGNATURE:
%   [rom, info] = ML_HNA(sys)
%   [rom, info] = ML_HNA(sys, opts)
%
% INPUT:
%   sys  - state-space object, containing the standard system's matrices
%          (see above 2))
%   opts - structure, containing optional parameters (see above 1))
%
% OUTPUT:
%   rom  - state-space object, containing the reduced-order standard system
%          (see above 2))
%   info - structure, containing information (see above 1))
%
%
% REFERENCE:
%   P. BENNER, E. S. QUINTANA-ORTI, G. E. S. QUINTANA-ORTI. Computing
%   optimal Hankel norm approximations of large-scale systems. In "2004
%   43rd IEEE Conference on Decision and Control (CDC)," volume 3, pages
%   3078-3083, Atlantis, Paradise Island, Bahamas, December 2004. Institute
%   of Electrical and Electronics Engineers.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'hankelsignmopts')
    assert(isa(opts.hankelsignmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.hankelsignmopts has to be a struct!');
else
    opts.hankelsignmopts = struct();
end

if ml_field_set_to_value(opts, 'hankelsylvopts')
    assert(isa(opts.hankelsylvopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.hankelsylvopts has to be a struct!');
else
    opts.hankelsylvopts = struct();
end

if ml_field_set_to_value(opts, 'lyapdlopts')
    assert(isa(opts.lyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapdlopts has to be a struct!');
else
    opts.lyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 2;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 2;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into the stable and anti-stable parts.
[sys, infoADTF] = ml_adtf(sys, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE MINIMAL, BALANCED REALIZATION.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the full-rank factors of the system Gramians.
if size(sys.A, 1) > 0
    [R, L, infoLYAPDL] = ml_lyapdl_sgn_fac(sys.A, sys.B, sys.C, [], ...
        opts.lyapdlopts);
else
    [R, L, infoLYAPDL] = deal([]);
end

[sys, hsv] = ml_sr(sys, R, L, 1, opts);

n  = size(sys.A, 1);
nh = ml_order(hsv, size(sys.Au, 1), 0, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TRANSFORMATION TO REDUCED-ORDER MODEL.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nh < n
    sigma = hsv(nh + 1);
    
    % Get multiplicity of the chosen Hankel singular value.
    k = 1;
    
    while k < n - nh
        if abs(sigma - hsv(nh + 1 + k)) > sigma * log(n) * sqrt(eps)
            break;
        end
        
        k = k + 1;
    end
    
    % Permutation of the system.
    if nh + k < n
        p = [1:nh, nh+k+1:n, nh+1:nh+k];
        
        sys.A = sys.A(p,p);
        sys.B = sys.B(p, :);
        sys.C = sys.C(: , p);
        hsvh  = hsv(p);
    else
        hsvh = hsv(1:n);
    end
    
    % Transformation to an all-pass error system.
    nk = n - k;
    
    U = sigma * pinv(sys.C(: , nk+1:n)') * sys.B(nk+1:n, :);
    G = sparse(1:nk, 1:nk, 1 ./ (hsvh(1:nk).^2 - sigma^2));
    S = sparse(1:nk, 1:nk, hsvh(1:nk));
    A = sys.A(1:nk, 1:nk);
    B = sys.B(1:nk, :);
    C = sys.C(: , 1:nk);
    
    rom.A = G * (sigma^2 * A' + S * A * S + C' * U * B');
    rom.B = G * (S * B - C' * U);
    rom.C = C * S - U * B';
    rom.D = sys.D + U;
    
    % Additive decomposition.
    hankelopts               = struct();
    hankelopts.stabsignmopts = opts.hankelsignmopts;
    hankelopts.stabsylvopts  = opts.hankelsylvopts;
    hankelopts.UnstabDim     = nk - nh;
    
    [rom, infoHANKEL] = ml_adtf(rom, hankelopts);
else
    sigma = 0;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(rom.A, sys.Au);
        varargout{2} = [rom.B; sys.Bu];
        varargout{3} = [rom.C, sys.Cu];
        varargout{4} = rom.D;
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(rom.A, sys.Au), ...
            'B', [rom.B; sys.Bu], ...
            'C', [rom.C, sys.Cu], ...
            'D', rom.D);
        
    case 2
        varargout{1} = ss(...
            blkdiag(rom.A, sys.Au), ...
            [rom.B; sys.Bu], ...
            [rom.C, sys.Cu], ...
            rom.D);
end

% Assign output information.
info = struct(...
    'AbsErrBound', 2 * sum(hsv(nh+1:end)), ...
    'Hsv'        , hsv, ...
    'infoADTF'   , infoADTF, ...
    'infoHASIGNM', infoHANKEL.infoSTABSIGNM, ...
    'infoHASYLV' , infoHANKEL.infoSTABSYLV, ...
    'infoLYAPDL' , infoLYAPDL, ...
    'Ns'         , n, ...
    'Nu'         , size(sys.Au, 1), ...
    'Sigma'      , sigma);

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
