function [varargout] = ml_mt(varargin)
%ML_MT Modal truncation for standard systems.
%
% DESCRIPTION:
%   This function computes the modal truncation for a standard system of
%   the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, a block diagonalization of the matrix A is performed using
%   the matrix sign function and a Sylvester equation, such that
%
%            [ Ar  0 ]                        [ Cr ]
%       A2 = [       ], B2 = [ Br, B1 ], C2 = [    ], Dr = D,
%            [ 0  A1 ]                        [ C1 ]
%
%   where Ar contains all the eigenvalues of A, which have a larger real
%   part than a given alpha. As result, the reduced-order system is given
%   by
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t).                                      (4)
%
%
% 1) If the standard system is given by matrices:
% SIGNATURE:
%   [Ar, Br, Cr, Dr, info] = ML_MT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_MT(A, B, C, D, opts)
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   opts - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | Alpha           | scalar, such that all finite eigenvalues with the |
%   |                 | real part smaller than Alpha are truncated        |
%   |                 | (default -1.0)                                    |
%   |-----------------|---------------------------------------------------|
%   | signmopts       | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition of the eigenvalues, with            |
%   |                 |   AbsTol    - tolerance for the absolute change   |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative change   |
%   |                 |               (default 1.0e+02*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | sylvopts        | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition of the eigenvalues, with            |
%   |                 |   AbsTol    - tolerance for the absolute error    |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative error    |
%   |                 |               (default 1.0e+01*n*eps)             |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   info - structure, containing the following information:
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | infoSIGNM       | structure, containing information about the       |
%   |                 | matrix sign function method, with                 |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSYLV        | structure, containing information about the       |
%   |                 | Sylvester equation solver, with                   |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | N               | Dimension of the reduced-order model              |
%   |-----------------|---------------------------------------------------|
%
% 2) If the standard system is given in a struct:
% SIGNATURE:
%   [rom, info] = ML_MT(sys)
%   [rom, info] = ML_MT(sys, opts)
%
% INPUT:
%   sys  - structure, containing the standard system's matrices
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (1) with dimensions n x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (2) with dimensions p x n             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (2) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   opts - structure, containing optional parameters (see above 1))
%
% OUTPUT:
%   rom  - structure, with the following entries
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (3) with dimensions r x r             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (3) with dimensions r x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (4) with dimensions p x r             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (4) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   info - structure, containing information (see above 1))
%
% 3) If the standard system is given as state-space object:
% SIGNATURE:
%   [rom, info] = ML_MT(sys)
%   [rom, info] = ML_MT(sys, opts)
%
% INPUT:
%   sys  - state-space object, containing the standard system's matrices
%          (see above 2))
%   opts - structure, containing optional parameters (see above 1))
%
% OUTPUT:
%   rom  - state-space object, containing the reduced-order standard system
%          (see above 2))
%   info - structure, containing information (see above 1))
%
%
% REFERENCE:
%   P. BENNER, E. S. QUINTANA-ORTI. Model reduction based on spectral
%   projection methods. In P. Benner, V. Mehrmann, and D. C. Sorensen,
%   "Dimension reduction of large-scale systems", chapter 1, pages 5-48.
%   Springer-Verlag, Berlin, Heidelberg, New York, 2005.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'Alpha')
    ml_assert_scalar(opts.Alpha, 'opts.Alpha');
else
    opts.Alpha = -1.0;
end

if ml_field_set_to_value(opts, 'signmopts')
    assert(isa(opts.signmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.signmopts has to be a struct!');
else
    opts.signmopts = struct();
end

if ml_field_set_to_value(opts, 'sylvopts')
    assert(isa(opts.sylvopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.sylvopts has to be a struct!');
else
    opts.sylvopts = struct();
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of desired and undesired eigenvalues.
n              = size(sys.A, 1);
[Z, infoSIGNM] = ml_signm(sys.A - opts.Alpha * eye(n), opts.signmopts);

n1 = (n + round(trace(Z))) / 2;

if (n1 > 0) && (n1 < n)
    [Q, ~, ~] = qr(eye(n) + Z);
    
    A1     = Q(: , n1+1:end)' * sys.A * Q(: , n1+1:end);
    Wa     = Q(: , 1:n1)' * sys.A * Q(: , n1+1:end);
    sys.A  = Q(: , 1:n1)' * sys.A * Q(: , 1:n1);
    
    [Y, infoSYLV] = ml_sylv_sgn(-sys.A + opts.Alpha * eye(n1), ...
        A1 - opts.Alpha * eye(n - n1), Wa, opts.sylvopts);
    
    sys.B = Q' * sys.B;
    sys.B = sys.B(1:n1, :) - Y * sys.B(n1+1:end, :);
    
    sys.C = sys.C * Q(: , 1:n1);
elseif n1 == n
    [sys.A, sys.B, sys.C, sys.D] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = sys.A;
        varargout{2} = sys.B;
        varargout{3} = sys.C;
        varargout{4} = sys.D;
        
    case 1
        varargout{1} = sys;
        
    case 2
        varargout{1} = ss(sys.A, sys.B, sys.C, sys.D);
end

% Assign output information.
info = struct(...
    'infoSIGNM', infoSIGNM, ...
    'infoSYLV' , infoSYLV, ...
    'N'        , size(sys.A, 1));

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
