function [varargout] = ml_mt_dss(varargin)
%ML_MT_DSS Modal truncation for descriptor systems.
%
% DESCRIPTION:
%   This function computes the generalized modal truncation for a
%   descriptor system of the form
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, a block diagonalization of the matrix pencil s*E - A is
%   performed using the matrix disk function, such that
%
%            [ Ei 0  0  ]       [ Ai 0  0  ]
%       E2 = [ 0  E0 0  ], A2 = [ 0  A0 0  ],
%            [ 0  0  E1 ]       [ 0  0  A1 ]
%
%                                 [ Ci ]
%       B2 = [ Bi, B0, B1 ], C2 = [ C0 ],
%                                 [ C1 ]
%
%   where the matrix pencil s*Ei - Ai contains all infinite eigenvalues
%   and s*E0 - A0 the finite eigenvalues with the real part larger than a
%   given alpha. As result, the reduced-order system is given by
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t),                                   (4)
%
%   with
%
%            [ E0  0 ]       [ A0  0 ]                        [ C0 ]
%       Er = [       ], A2 = [       ], B2 = [ B0, Bi ], C2 = [    ].
%            [ 0  Ei ]       [ 0  Ai ]                        [ Ci ]
%
%
% 1) If the descriptor system is given by matrices:
% SIGNATURE:
%   [Ar, Br, Cr, Dr, Er, info] = ML_MT_DSS(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_MT_DSS(A, B, C, D, E, opts)
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   opts - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | Alpha           | scalar, such that all finite eigenvalues with the |
%   |                 | real part smaller than Alpha are truncated        |
%   |                 | (default -1.0)                                    |
%   |-----------------|---------------------------------------------------|
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   |-----------------|---------------------------------------------------|
%   | diskopts        | structure, containing the optional parameters for |
%   |                 | the inverse free iteration of the disk function   |
%   |                 | method, with                                      |
%   |                 |   AbsTol  - tolerance for the absolute change     |
%   |                 |             (default 0)                           |
%   |                 |   Info    - disable/enable display of verbose     |
%   |                 |             status information during iteration   |
%   |                 |             (default 0)                           |
%   |                 |   MaxIter - maximum number of iteration steps     |
%   |                 |             (default 100)                         |
%   |                 |   RelTol  - tolerance for the relative change     |
%   |                 |             (default 1.0e+02*n*eps)               |
%   |-----------------|---------------------------------------------------|
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, with                           |
%   |                 |   AbsTol    - tolerance for the absolute change   |
%   |                 |               (default 0)                         |
%   |                 |   Dimension - dimension of the deflating subspace |
%   |                 |               corresponding to the infinite       |
%   |                 |               eigenvalues, negative if unknown    |
%   |                 |               (default -1)                        |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RankTol   - tolerance used in numerical rank    |
%   |                 |               detection                           |
%   |                 |               (default  log(n)*eps)               |
%   |                 |   RelTol    - tolerance for the relative change   |
%   |                 |               (default 1.0e+02*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | RankTol         | nonnegative scalar, tolerance used for the        |
%   |                 | determination of deflating subspaces              |
%   |                 | (default log(n)*eps)                              |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   info - structure, containing the following information:
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | infoDISK        | structure, containing information about the disk  |
%   |                 | function method used for the modal truncation of  |
%   |                 | the system, with                                  |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoINFDISK     | structure, containing information about the disk  |
%   |                 | function method used for the separation of the    |
%   |                 | infinite part, with                               |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | N               | Dimension of the finite part in the reduced-order |
%   |                 | model                                             |
%   |-----------------|---------------------------------------------------|
%   | Ni              | Dimension of the infinite part in the reduced-    |
%   |                 | order model                                       |
%   |-----------------|---------------------------------------------------|
%
% 2) If the descriptor system is given in a struct:
% SIGNATURE:
%   [rom, info] = ML_MT_DSS(sys)
%   [rom, info] = ML_MT_DSS(sys, opts)
%
% INPUT:
%   sys  - structure, containing the descriptor system's matrices
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (1) with dimensions n x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (2) with dimensions p x n             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (2) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   |        E        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   opts - structure, containing optional parameters (see above 1))
%
% OUTPUT:
%   rom  - structure, containing the reduced-order descriptor system
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (3) with dimensions r x r             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (3) with dimensions r x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (4) with dimensions p x r             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (4) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   |        E        | matrix from (3) with dimensions r x r             |
%   |-----------------|---------------------------------------------------|
%   info - structure, containing information (see above 1))
%
% 3) If the descriptor system is given as state-space object:
% SIGNATURE:
%   [rom, info] = ML_MT_DSS(sys)
%   [rom, info] = ML_MT_DSS(sys, opts)
%
% INPUT:
%   sys  - state-space object, containing the descriptor system's matrices
%          (see above 2))
%   opts - structure, containing optional parameters (see above 1))
%
% OUTPUT:
%   rom  - state-space object, containing the reduced-order descriptor
%          system (see above 2))
%   info - structure, containing information (see above 1))
%
%
% REFERENCE:
%   P. BENNER, E. S. QUINTANA-ORTI. Model reduction based on spectral
%   projection methods. In P. Benner, V. Mehrmann, and D. C. Sorensen,
%   "Dimension reduction of large-scale systems," chapter 1, pages 5-48.
%   Springer-Verlag, Berlin, Heidelberg, New York, 2005.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_lti2struct_dss(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'Alpha')
    ml_assert_scalar(opts.Alpha, 'opts.Alpha');
else
    opts.Alpha = -1.0;
end

if ml_field_set_to_value(opts, 'diskopts')
    assert(isa(opts.diskopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.diskopts has to be a struct!');
else
    opts.diskopts = struct();
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into finite and infinite parts.
opts.stabdec.Dimension = 0;
[sys, infoGADTF]       = ml_adtf_dss(sys, opts);

n = size(sys.A, 1);

% Decomposition of desired and undesired eigenvalues.
[Aspace, Espace, infoDISK] = ml_disk(sys.A + (1 - opts.Alpha) * sys.E, ...
    sys.A - (1 + opts.Alpha) * sys.E, opts.diskopts);
[Q, ~, n1]                 = ml_getqz(sys.A, sys.E, Aspace, opts);

if (n1 > 0) && (n1 < n)
    opts.Dimension = n - n1;
    [~, V, ns]     = ml_getqz(sys.A, sys.E, Espace, opts);
    
    sys.A = Q(: , n1+1:end)' * sys.A * V(: , 1:ns);
    sys.E = Q(: , n1+1:end)' * sys.E * V(: , 1:ns);
    sys.B = Q(: , n1+1:end)' * sys.B;
    sys.C = sys.C * V(: , 1:ns);
elseif n1 == n
    [sys.A, sys.B, sys.C, sys.D, sys.E] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(sys.A, sys.Ainf);
        varargout{2} = [sys.B; sys.Binf];
        varargout{3} = [sys.C, sys.Cinf];
        varargout{4} = sys.D;
        varargout{5} = blkdiag(sys.E, sys.Einf);
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(sys.A, sys.Ainf), ...
            'B', [sys.B; sys.Binf], ...
            'C', [sys.C, sys.Cinf], ...
            'D', sys.D, ...
            'E', blkdiag(sys.E, sys.Einf));
        
    case 2
        varargout{1} = dss(...
            blkdiag(sys.A, sys.Ainf), ...
            [sys.B; sys.Binf], ...
            [sys.C, sys.Cinf], ...
            sys.D, ...
            blkdiag(sys.E, sys.Einf));
end

% Assign output information.
info = struct(...
    'infoDISK'   , infoDISK, ...
    'infoINFDISK', infoGADTF.infoINFDISK, ...
    'N'          , size(sys.A, 1), ...
    'Ni'         , size(sys.Ainf, 1));

if ioformat == 0
    varargout{6} = info;
else
    varargout{2} = info;
end
