function [Q, Z, nu] = ml_getqz(A, E, Aspace, opts)
%ML_GETQZ Subspace extraction method.
%
% DESCRIPTION:
%   Computes orthogonal transformation matrices Q and Z for a deflating
%   subspace of the matrix pencil s*E - A, where the basis is given by the
%   null space of the matrix Aspace. The transformation Q'*(s*E - A)*Z is
%   in block triangular form with the leading block corresponding to the
%   deflating subspace.
%
% SIGNATURE:
%   [Q, Z, nu] = ML_GETQZ(A, E, Aspace)
%   [Q, Z, nu] = ML_GETQZ(A, E, Aspace, opts)
%
% INPUT:
%   A      - a matrix with dimensions n x n
%   E      - a matrix with dimensions n x n
%   Aspace - a matrix with dimensions n x n, with its null space the
%            deflating subspace
%   opts   - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | Dimension       | integer, dimension of the deflating subspace,     |
%   |                 | negative if unknown                               |
%   |                 | (default -1)                                      |
%   |-----------------|---------------------------------------------------|
%   | RankTol         | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of Aspace to determine the |
%   |                 | rank of Aspace                                    |
%   |                 | (default log(n)*eps)                              |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   Q  - right orthogonal transformation matrix onto the deflating subspace
%   Z  - left orthogonal transformation matrix onto the deflating subspace
%   nu - dimension of the deflating subspace
%
%
% REFERENCE:
%   P. BENNER. Partial stabilization of descriptor systems using spectral
%   projectors. In P. Van Dooren, P. S. Bhattacharyya, H. R. Chan,
%   V. Olshevsky, and A. Routray, "Numerical Linear Algebra in Signals,
%   Systems and Control," chapter 3, pages 55-76. Springer Netherlands,
%   Dordrecht, 2011.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(3, 4);

if (nargin < 4) || isempty(opts)
    opts = struct();
end

% Check input matrices.
n = size(A, 1);

assert(isequal(size(A), [n n]), ...
    'MORLAB:data', ...
    'The matrix A has to be square!');

assert(isequal(size(E), [n n]), ...
    'MORLAB:data', ...
    'The matrix E must have the same dimensions as A!');

assert(isequal(size(Aspace), [n n]), ...
    'MORLAB:data', ...
    'The matrix E must have the same dimensions as A!');

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'RankTol')
    ml_assert_nonnegscalar(opts.RankTol, 'opts.RankTol');
else
    opts.RankTol = log(n) * eps;
end

if ml_field_set_to_value(opts, 'Dimension')
    ml_assert_integer(opts.Dimension, 'opts.Dimension');
else
    opts.Dimension = -1;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBSPACE EXTRACTION METHOD.                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if opts.Dimension < 0
    [~, S, V] = svd(Aspace);
    
    if S(1,1) <= opts.RankTol
        r = 0;
    else
        r = sum(diag(S) > S(1,1) * opts.RankTol);
    end
else
    [V, ~, ~] = qr(Aspace');
    r         = n - opts.Dimension;
end

Z = V(: , r+1:n);

[Q, ~, ~] = qr([A * Z, E * Z]);
Z         = [Z, V(:, 1:r)];
nu        = n - r;
