function [sys, opts, ioformat] = ml_lti2struct(varargin)
%ML_LTI2STRUCT Maps given input to a checked standard LTI system struct.
%
% DESCRIPTION:
%   This function gets the possible inputs of the model order reduction
%   methods, checks the system matrices and returns structs for the system
%   and the options in the right format
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Also, the input format is returned as an integer.
%
% 1) If the standard system is given by matrices:
% SIGNATURE:
%   [sys, opts, ioformat] = ML_LTI2STRUCT(A, B, C, D)
%   [sys, opts, ioformat] = ML_LTI2STRUCT(A, B, C, D, opts)
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   opts - structure, containing optional parameters
%
% 2) If the standard system is given in a struct:
% SIGNATURE:
%   [sys, opts, ioformat] = ML_LTI2STRUCT(sys)
%   [sys, opts, ioformat] = ML_LTI2STRUCT(sys, opts)
%
% INPUT:
%   sys  - structure, containing the standard system's matrices
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (1) with dimensions n x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (2) with dimensions p x n             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (2) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   opts - structure, containing optional parameters
%
% 3) If the standard system is given as state-space object:
% SIGNATURE:
%   [sys, opts, ioformat] = ML_LTI2STRUCT(sys)
%   [sys, opts, ioformat] = ML_LTI2STRUCT(sys, opts)
%
% INPUT:
%   sys  - state-space object, containing the standard system's matrices
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (1) with dimensions n x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (2) with dimensions p x n             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (2) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   opts - structure, containing optional parameters
%
%
% OUTPUT:
%   sys      - structure, containing the standard system's matrices
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   |        A        | matrix from (1) with dimensions n x n             |
%   |-----------------|---------------------------------------------------|
%   |        B        | matrix from (1) with dimensions n x m             |
%   |-----------------|---------------------------------------------------|
%   |        C        | matrix from (2) with dimensions p x n             |
%   |-----------------|---------------------------------------------------|
%   |        D        | matrix from (2) with dimensions p x m             |
%   |-----------------|---------------------------------------------------|
%   opts     - structure, containing optional parameters
%   ioformat - integer, describing the input format
%                0 - matrices, see 1)
%                1 - structs, see 2)
%                2 - state-space objects, see 3)

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%

if isa(varargin{1}, 'double') % Input case: matrices.
    narginchk(4, 5)
    
    if (nargin < 5) || isempty(varargin{5})
        opts = struct();
    else
        opts = varargin{5};
    end
    
    % Assign and check standard system.
    sys = struct(...
        'A', varargin{1}, ...
        'B', varargin{2}, ...
        'C', varargin{3}, ...
        'D', varargin{4});
    
    ml_assert_stdsys(sys);
    
    if ml_field_set_to_value(sys, 'D')
        assert(isequal(size(sys.D), [size(sys.C, 1), size(sys.B, 2)]), ...
            'MORLAB:data', ...
            'The matrix D must have the dimensions %d x %d!', ...
            size(sys.C, 1), size(sys.B, 2));
    end
    
    % Output format.
    ioformat = 0;
elseif isa(varargin{1}, 'struct') % Input case: structs.
    narginchk(1, 2);
    
    if (nargin < 2) || isempty(varargin{2})
        opts = struct();
    else
        opts = varargin{2};
    end
    
    % Check input standard system.
    sys = varargin{1};
    
    ml_assert_stdsys(sys);
    
    if ml_field_set_to_value(sys, 'D')
        assert(isequal(size(sys.D), [size(sys.C, 1), size(sys.B, 2)]), ...
            'MORLAB:data', ...
            'The matrix D must have the dimensions %d x %d!', ...
            size(sys.C, 1), size(sys.B, 2));
    else
        sys.D = [];
    end
    
    % Output format.
    ioformat = 1;
elseif isa(varargin{1}, 'ss') % Input case: ss.
    narginchk(1, 2);
    
    if (nargin < 2) || isempty(varargin{2})
        opts = struct();
    else
        opts = varargin{2};
    end
    
    % Assign input standard system.
    sys = struct(...
        'A', varargin{1}.A, ...
        'B', varargin{1}.B, ...
        'C', varargin{1}.C, ...
        'D', varargin{1}.D);
    
    % Output format.
    ioformat = 2;
else % Input case: invalid type.
    error('MORLAB:data', ...
        'Such type of input data is not implemented!');
end
