function [opts] = ml_morlabopts(fname)
%ML_MORLABOPTS Option struct generator for MORLAB functions.
%
% DESCRIPTION:
%   This function can be used to generate an option structure for the
%   MORLAB function specified by fname. The structure is filled with empty
%   arrays such that the default values of the MORLAB function are called.
%   For the choice of the default values, call this function without an
%   output argument or see the documentation of the corresponding function.
%
% SIGNATURE:
%   ML_MORLABOPTS(fname)
%   opts = ML_MORLABOPTS(fname)
%
% INPUT:
%   fname - character array, the name of the function for which the option
%           struct is requested
%
% OUTPUT:
%   opts - struct, the desired option structure for a MORLAB function

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUT.                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

assert(ischar(fname), ...
    'MORLAB:data', ...
    'The given function name has to be a character array.');

% Check if hyperlinks can be used.
hasHyperlink = usejava('desktop');

% Simplify case checks by restricting to lower case letters.
fname = lower(fname);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHOOSE FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign option structs.
switch fname
    % Check functions.
    case {'ml_assert_boolean', ...
            'ml_assert_descsys', ...
            'ml_assert_infdescsys', ...
            'ml_assert_integer', ...
            'ml_assert_nonnegint', ...
            'ml_assert_nonnegintinf', ...
            'ml_assert_nonnegscalar', ...
            'ml_assert_nonnegvector', ...
            'ml_assert_posinteger', ...
            'ml_assert_posscalar', ...
            'ml_assert_scalar', ...
            'ml_assert_stdsys', ...
            'ml_field_set_to_value'}
        fprintf(1, [' The requested function doesn''t use an ' ...
            'option struct!']);
        fprintf(1, '\n');
        if nargout > 0
            opts = struct();
        end
        
    % Equation solver routines.
    case {'ml_abe_sgn', ...
            'ml_lyap_sgn', ...
            'ml_sylv_sgn'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_care_nwt_fac'
        if nargout == 0
            fprintf(1, ['   AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['     Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['       K0: [ matrix, s*E - (A - BK0) Hurwitz ' ...
                '| {zeros(m, n)} ]\n']);
            if hasHyperlink
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['  MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['   RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol'  , [], ...
                'Info'    , [], ...
                'K0'      , [], ...
                'lyapopts', ml_morlabopts('ml_lyap_sgn_fac'), ...
                'MaxIter' , [], ...
                'RelTol'  , []);
        end
        
    case 'ml_gdlyap_smith_fac'
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['   Index: [ integer | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'Index'  , [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case {'ml_lyap_sgn_fac', ...
            'ml_lyapdl_sgn_fac'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, [' CompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'CompTol', [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_pcare_nwt_fac'
        if nargout == 0
            fprintf(1, ['   AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['  CompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['     Info: [ 1 | ' ...
                '{0} ]\n']);
            if hasHyperlink
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['  MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['   RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol'  , [], ...
                'CompTol' , [], ...
                'Info'    , [], ...
                'lyapopts', ml_morlabopts('ml_lyap_sgn_fac'), ...
                'MaxIter' , [], ...
                'RelTol'  , []);
        end
        
    % Standard model order reduction routines.
    case 'ml_brbt'
        if nargout == 0
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    case 'ml_bst'
        if nargout == 0
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            if hasHyperlink
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'Epsilon'         , [], ...
                'lyapopts'        , ml_morlabopts('ml_lyap_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_bt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_hinfbt'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['            Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            temp = ml_morlabopts('ml_care_nwt_fac');
            temp = rmfield(temp, 'K0');
            
            opts = struct(...
                'Beta'            , [], ...
                'careopts'        , temp, ...
                'Gamma'           , [], ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'StabMethod'      , [], ...
                'stabmethodopts'  , ml_morlabopts('ml_abe_sgn'), ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_hna'
        if nargout == 0
            if hasHyperlink
                fprintf(1, ['  hankelsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">'...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['   hankelsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['  hankelsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['   hankelsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['        MinRelTol: [ nonnegative scalar | ' ...
                '{log(length(Hsv))*eps} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'hankelsignmopts' , ml_morlabopts('ml_signm'),...
                'hankelsylvopts'  , ml_morlabopts('ml_sylv_sgn'), ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'MinRelTol'       , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_lqgbt'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            temp = ml_morlabopts('ml_care_nwt_fac');
            temp = rmfield(temp, 'K0');
            
            opts = struct(...
                'Beta'            , [], ...
                'careopts'        , temp, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'StabMethod'      , [], ...
                'stabmethodopts'  , ml_morlabopts('ml_abe_sgn'), ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_mt'
        if nargout == 0
            fprintf(1, ['     Alpha: [ real scalar | ' ...
                '{-1.0} ]\n']);
            if hasHyperlink
                fprintf(1, [' signmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['  sylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, [' signmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['  sylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
        else
            opts = struct(...
                'Alpha'     , [], ...
                'signmopts' , ml_morlabopts('ml_signm'), ...
                'sylvopts'  , ml_morlabopts('ml_sylv_sgn'));
        end
        
    case 'ml_prbt'
        if nargout == 0
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv))} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Epsilon'         , [], ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    % Descriptor model order reduction routines.
    case 'ml_brbt_dss'
        if nargout == 0
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_adtf_dss');
            
            opts = struct(...
                'DecompTol'       , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    case 'ml_bst_dss'
        if nargout == 0
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_adtf_dss');
            
            opts = struct(...
                'DecompTol'       , temp.DecompTol, ...
                'Epsilon'         , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'lyapopts'        , ml_morlabopts('ml_lyap_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'stabdecopts'     , temp.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_bt_dss'
        if nargout == 0
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_adtf_dss');
            
            opts = struct(...
                'DecompTol'       , temp.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_hinfbt_dss'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp1 = ml_morlabopts('ml_adtf_dss');
            temp2 = ml_morlabopts('ml_partstab_dss');
            temp3 = ml_morlabopts('ml_care_nwt_fac');
            temp3 = rmfield(temp3, 'K0');
            
            opts = struct(...
                'Beta'            , temp2.Beta, ...
                'careopts'        , temp3, ...
                'DecompTol'       , temp1.DecompTol, ...
                'Gamma'           , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp1.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp1.stabdecopts, ...
                'StabMethod'      , temp2.StabMethod, ...
                'stabmethodopts'  , temp2.stabmethodopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_hna_dss'
        if nargout == 0
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
                fprintf(1, ['   hankeldiskopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
                fprintf(1, ['   hankeldiskopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['        MinRelTol: [ nonnegative scalar | ' ...
                '{log(length(Hsv))*eps} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp1 = ml_morlabopts('ml_adtf_dss');
            temp2 = rmfield(temp1.stabdecopts, 'Dimension');
            
            opts = struct(...
                'DecompTol'       , temp1.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'hankeldiskopts'  , temp2, ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp1.infdecopts, ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'MinRelTol'       , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp1.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_lqgbt_dss'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp1 = ml_morlabopts('ml_adtf_dss');
            temp2 = ml_morlabopts('ml_partstab_dss');
            temp3 = ml_morlabopts('ml_care_nwt_fac');
            temp3 = rmfield(temp3, 'K0');
            
            opts = struct(...
                'Beta'            , temp2.Beta, ...
                'careopts'        , temp3, ...
                'DecompTol'       , temp1.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp1.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp1.stabdecopts, ...
                'StabMethod'      , temp2.StabMethod, ...
                'stabmethodopts'  , temp2.stabmethodopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_mt_dss'
        if nargout == 0
            fprintf(1, ['      Alpha: [ real scalar | ' ...
                '{-1.0} ]\n']);
            fprintf(1, ['  DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['   diskopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>} ]\n']);
                fprintf(1, [' infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['   diskopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk'')} ]\n']);
                fprintf(1, [' infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['    RankTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
        else
            temp = ml_morlabopts('ml_adtf_dss');
            
            opts = struct(...
                'Alpha'     , [], ...
                'DecompTol' , temp.DecompTol, ...
                'diskopts'  , ml_morlabopts('ml_disk'), ...
                'infdecopts', temp.infdecopts, ...
                'RankTol'   , []);
        end
        
    case 'ml_prbt_dss'
        if nargout == 0
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_adtf_dss');
            
            opts = struct(...
                'DecompTol'       , temp.DecompTol, ...
                'Epsilon'         , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    % Subroutines.
    case 'ml_adtf'
        if nargout == 0
            if hasHyperlink
                fprintf(1, [' stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['  stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, [' stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['  stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['     UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'stabsignmopts', ml_morlabopts('ml_signm'), ...
                'stabsylvopts' , ml_morlabopts('ml_sylv_sgn'), ...
                'UnstabDim'    , []);
        end
        
    case {'ml_bfsr', 'ml_sr'}
        if nargout == 0
            fprintf(1, ['     Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, [' MinRelTol: [ nonnegative scalar | ' ...
                '{log(length(Hsv))*eps} ]\n']);
            fprintf(1, ['     Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu} ]\n']);
            fprintf(1, [' Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Gamma'    , [], ...
                'MinRelTol', [], ...
                'Order'    , [], ...
                'Tolerance', []);
        end
        
    case {'ml_lti2struct_dss', 'ml_lti2struct'}
        fprintf(1, [' The requested function is an input parser ' ...
            'and can\n take arbitrary option structs!\n']);
        if nargout > 0
            opts = struct();
        end
        
    case {'ml_disk', 'ml_signm'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_adtf_dss'
        if nargout == 0
            fprintf(1, ['   DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['  infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, [' stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['  infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, [' stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
        else
            temp           = ml_morlabopts('ml_disk');
            temp.Dimension = [];
            temp.RankTol   = [];
            temp           = orderfields(temp);
            
            opts = struct(...
                'DecompTol'  , [], ...
                'infdecopts' , temp, ...
                'stabdecopts', temp);
        end
        
    case {'ml_bfsr_dss', 'ml_sr_dss'}
        if nargout == 0
            fprintf(1, ['         Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, [' ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['         Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['     MinRelTol: [ nonnegative scalar | ' ...
                '{log(length(Hsv))*eps} ]\n']);
            fprintf(1, ['         Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, ['     Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Gamma'        , [], ...
                'ImproperTrunc', [], ...
                'Index'        , [], ...
                'MinRelTol'    , [], ...
                'Order'        , [], ...
                'Tolerance'    , []);
        end
        
    case 'ml_getqz'
        if nargout == 0
            fprintf(1, [' Dimension: [ integer | ' ...
                '{-1} ]\n']);
            fprintf(1, ['   RankTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
        else
            opts = struct(...
                'Dimension', [], ...
                'RankTol'  , []);
        end
        
    case 'ml_partstab_dss'
        if nargout == 0
            fprintf(1, ['           Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            fprintf(1, ['      DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['     infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['    stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['     infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['    stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['     StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
            else
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
            end
        else
            temp = ml_morlabopts('ml_adtf_dss');
            
            opts = struct(...
                'Beta'          , [], ...
                'DecompTol'     , temp.DecompTol, ...
                'infdecopts'    , temp.infdecopts, ...
                'stabdecopts'   , temp.stabdecopts, ...
                'StabMethod'    , [], ...
                'stabmethodopts', ml_morlabopts('ml_abe_sgn'));
        end
        
    case {'ml_morlabopts', ...
            'ml_sigmaplot'}
        fprintf(1, [' The requested function doesn''t use ' ...
            'an option struct!']);
        fprintf(1, '\n');
        if nargout > 0
            opts = struct();
        end
        
    case 'ml_order'
        if nargout == 0
            fprintf(1, ['     Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, [' MinRelTol: [ nonnegative scalar | ' ...
                '{log(length(Hsv))*eps} ]\n']);
            fprintf(1, ['     Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu [+ Ni]} ]\n']);
            fprintf(1, [' Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Gamma'    , [], ...
                'MinRelTol', [], ...
                'Order'    , [], ...
                'Tolerance', []);
        end
        
    case 'ml_partstab'
        if nargout == 0
            fprintf(1, ['           Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            fprintf(1, ['     StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
                fprintf(1, ['  stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
            else
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
                fprintf(1, ['  stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
            end
            fprintf(1, ['      UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'Beta'          , [], ...
                'StabMethod'    , [], ...
                'stabmethodopts', ml_morlabopts('ml_lyap_sgn'), ...
                'stabsignmopts' , ml_morlabopts('ml_signm'), ...
                'UnstabDim'     , []);
        end
        
    % Non-MORLAB functions.
    otherwise
        error('MORLAB:data', ...
            'The given function name is not implemented in MORLAB.');
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DEFAULT HELP LINE.                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Default help link for more details.
if nargout == 0
    fprintf(1, '\n');
    if hasHyperlink
        fprintf(1, [' For more details see <a href="matlab:help ' ...
            '%s">%s</a>.\n'], fname, fname);
    else
        fprintf(1, ' For more details see %s.', fname);
    end
    fprintf(1, '\n');
end
