function [K, info] = ml_partstab(A, B, opts)
%ML_PARTSTAB Stabilizing feedback for standard systems.
%
% DESCRIPTION:
%   Partial stabilization is used for the continuous-time system of
%   odinary differential equations
%
%       x'(t) = Ax(t) + Bu(t)                                           (1)
%
%   to get a stabilizing feedback term K, such that all eigenvalues of
%   A - B*K are in the open left half-plane.
%
% SIGNATURE:
%   [K, info] = ML_PARTSTAB(A, B)
%   [K, info] = ML_PARTSTAB(A, B, opts)
%
% INPUT:
%   A    - matrix with dimensions n x n in (1)
%   B    - matrix with dimensions n x m in (1)
%   opts - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | Beta            | nonnegative scalar, used as shift of the          |
%   |                 | in Bass' algorithm for better conditioning if     |
%   |                 | StabMethod == 'lyap' is chosen                    |
%   |                 | (default 0.1)                                     |
%   |-----------------|---------------------------------------------------|
%   | StabMethod      | character array, determining algorithm for the    |
%   |                 | partial stabilization                             |
%   |                 |  'abe'  - algebraic Bernoulli equation based      |
%   |                 |  'lyap' - Bass' algorithm (Lyapunov equation)     |
%   |                 | (default 'abe')                                   |
%   |-----------------|---------------------------------------------------|
%   | stabmethodopts  | structure, containing the optional parameters for |
%   |                 | the sign function based Lyapunov or Bernoulli     |
%   |                 | equation solver used for the stabilization, with  |
%   |                 |   AbsTol  - tolerance for the absolute error      |
%   |                 |             (default 0)                           |
%   |                 |   Info    - disable/enable display of verbose     |
%   |                 |             status information during iteration   |
%   |                 |             (default 0)                           |
%   |                 |   MaxIter - maximum number of iteration steps     |
%   |                 |             (default 100)                         |
%   |                 |   RelTol  - tolerance for the relative error      |
%   |                 |             (default 1.0e+01*n*eps)               |
%   |-----------------|---------------------------------------------------|
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, with                                       |
%   |                 |   AbsTol  - tolerance for the absolute change     |
%   |                 |             (default 0)                           |
%   |                 |   Info    - disable/enable display of verbose     |
%   |                 |             status information during iteration   |
%   |                 |             (default 0)                           |
%   |                 |   MaxIter - maximum number of iteration steps     |
%   |                 |             (default 100)                         |
%   |                 |   RelTol  - tolerance for the relative change     |
%   |                 |             (default 1.0e+02*n*eps)               |
%   |-----------------|---------------------------------------------------|
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   K    - stabilizing feedback matrix of dimensions m x n
%   info - structure, containing the following information about the
%          partial stabilization method
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | infoSTABMETH    | structure, containing information about the sign  |
%   |                 | function based solver used for the stabilization, |
%   |                 | with                                              |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABMETH2   | structure, containing information about the sign  |
%   |                 | function based solver used for a second           |
%   |                 | stabilization if necessary, with                  |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABSIGNM   | structure, containing information about the       |
%   |                 | matrix sign function method used for the          |
%   |                 | decomposition, with                               |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABSIGNM2  | structure, containing information about the       |
%   |                 | matrix sign function method used for the a second |
%   |                 | decomposition, with                               |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | Method          | character array, shortcut of the used             |
%   |                 | stabilization algorithm, with 'abe' for the       |
%   |                 | algebraic Bernoulli equation and 'lyap' for Bass' |
%   |                 | algorithm                                         |
%   |-----------------|---------------------------------------------------|
%   | Ns              | Number of identified stable eigenvalues           |
%   |-----------------|---------------------------------------------------|
%   | Nu              | Number of identified anti-stable eigenvalues      |
%   |-----------------|---------------------------------------------------|
%
%
% REFERENCE:
%   P. BENNER. Partial stabilization of of descriptor systems using
%   spectral projectors. In P. Van Dooren, P. S. Bhattacharyya, H. R. Chan,
%   V. Olshevsky, and A. Routray, "Numerical Linear Algebra in Signals,
%   Systems and Control," chapter 3, pages 55-76. Springer Netherlands,
%   Dordrecht, 2011.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(2, 3);

if (nargin < 3) || isempty(opts)
    opts = struct();
end

% Check input matrices.
n = size(A, 1);
m = size(B, 2);

assert(isequal(size(A), [n n]), ...
    'MORLAB:data', ...
    'The matrix A has to be square!');

assert(size(B, 1) == n, ...
    'MORLAB:data', ...
    'The matrix B must have the same number of rows as A!');

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'Beta')
    ml_assert_nonnegscalar(opts.Beta, 'opts.Beta');
else
    opts.Beta = 0.1;
end

if ml_field_set_to_value(opts, 'stabsignmopts')
    assert(isa(opts.stabsignmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabsignmopts has to be a struct!');
else
    opts.stabsignmopts = struct();
end

if ml_field_set_to_value(opts, 'stabmethodopts')
    assert(isa(opts.stabmethodopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabmethodopts has to be a struct!');
else
    opts.stabmethodopts = struct();
end

if ml_field_set_to_value(opts, 'StabMethod')
    opts.StabMethod = lower(opts.StabMethod);
    
    assert(strcmp(opts.StabMethod, 'abe') ...
        || strcmp(opts.StabMethod, 'lyap'), ...
        'MORLAB:data', ...
        'The given stabilization method is not implemented!');
else
    opts.StabMethod = 'abe';
end

if ml_field_set_to_value(opts, 'UnstabDim')
    ml_assert_integer(opts.UnstabDim, 'opts.UnstabDim');
else
    opts.UnstabDim = -1;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION: ANTI-STABLE PART.                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition into stable and anti-stable parts.
if opts.UnstabDim == 0
    ns            = n;
    infoSTABSIGNM = [];
else
    [Z, infoSTABSIGNM] = ml_signm(A, opts.stabsignmopts);
    
    if opts.UnstabDim > 0
        ns = n - opts.UnstabDim;
    else
        ns = 0.5 * (n - round(trace(Z)));
    end
end

if (ns > 0) && (ns < n)
    [Q, ~, ~] = qr(eye(n) - Z);
    
    A = Q(: , ns+1:end)' * A * Q(: , ns+1:end);
    B = Q(: , ns+1:end)' * B;
elseif ns == n
    [A, B] = deal([]);
    Q      = speye(n);
else
    Q = speye(n);
end

nu = n - ns;

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION (I).                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch opts.StabMethod
    case 'lyap' % Bass' Algorithm
        [X, infoSTABMETH] = ml_lyap_sgn(-(A + opts.Beta * eye(nu))', ...
            2 * (B * B'), [], opts.stabmethodopts);
        K1                = B' * pinv(X);
        
    case 'abe' % Algebraic Bernoulli stabilization
        [X, infoSTABMETH] = ml_abe_sgn(A, B * B', [], opts.stabmethodopts);
        K1                = B' * X;
end

% Test of stability.
A              = A - B * K1;
opts.UnstabDim = sum(real(eig(A)) > 0);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION (II).                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if opts.UnstabDim > 0
    [Z, infoSTABSIGNM2] = ml_signm(A, opts.stabsignmopts);
    ns2                 = 0.5 * (nu - round(trace(Z)));
    
    if (ns2 > 0) && (ns2 < nu)
        [Q2, ~, ~] = qr(eye(nu) - Z);
        
        A = Q2(: , ns2+1:end)' * A * Q2(: , ns2+1:end);
        B = Q2(: , ns2+1:end)' * B;
    elseif (ns2 == 0) || (ns2 == nu)
        error('MORLAB:partstab', ...
            'Partial stabilization of the system failed!');
    end
    
    switch opts.StabMethod
        case 'lyap' % Bass' Algorithm
            opts.Beta          = 2 * norm(A, 'fro');
            [X, infoSTABMETH2] = ml_lyap_sgn(-(A + opts.Beta * ...
                eye(nu - ns2))', 2 * (B * B'), [], opts.stabmethodopts);
            K2                 = B' * pinv(X);
        case 'abe' % Algebraic Bernoulli stabilization
            [X, infoSTABMETH2] = ml_abe_sgn(A, B * B', [], ...
                opts.stabmethodopts);
            K2                 = B' * X;
    end
    
    if sum(real(eig(A - B * K2)) > 0)
        error('MORLAB:partstab', ...
            'Partial stabilization of the system failed!');
    end
    
    K = [zeros(m, ns), K1 + [zeros(m, ns2), K2] * Q2' ] * Q';
else
    K = [zeros(m, ns), K1] * Q';
    
    [infoSTABMETH2, infoSTABSIGNM2] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN INFORMATION.                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Collection of all information.
info = struct(...
    'infoSTABMETH'  , infoSTABMETH, ...
    'infoSTABMETH2' , infoSTABMETH2, ...
    'infoSTABSIGNM' , infoSTABSIGNM, ...
    'infoSTABSIGNM2', infoSTABSIGNM2, ...
    'Method'        , opts.StabMethod, ...
    'Ns'            , ns, ...
    'Nu'            , nu);
