function [K, info] = ml_partstab_dss(A, B, E, opts)
%ML_PARTSTAB_DSS Stabilizing feedback for descriptor systems.
%
% DESCRIPTION:
%   Partial stabilization is used for the continuous-time system of
%   differential-algebraic equations
%
%       Ex'(t) = Ax(t) + Bu(t)                                          (1)
%
%   to get a stabilizing feedback term K, such that all finite eigenvalues
%   of s*E - A + B*K are in the left open half-plane.
%
% SIGNATURE:
%   [K, info] = ML_PARTSTAB_DSS(A, B, E)
%   [K, info] = ML_PARTSTAB_DSS(A, B, E, opts)
%
% INPUT:
%   A    - matrix with dimensions n x n in (1)
%   B    - matrix with dimensions n x m in (1)
%   E    - matrix with dimensions n x n in (1)
%   opts - structure, containing the following optional entries:
%   |-----------------|---------------------------------------------------|
%   |    PARAMETER    |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | Beta            | nonnegative scalar, used as shift of the          |
%   |                 | in Bass' algorithm for better conditioning if     |
%   |                 | StabMethod == 'lyap' is chosen                    |
%   |                 | (default 0.1)                                     |
%   |-----------------|---------------------------------------------------|
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   |-----------------|---------------------------------------------------|
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, with                           |
%   |                 |   AbsTol    - tolerance for the absolute change   |
%   |                 |               (default 0)                         |
%   |                 |   Dimension - dimension of the deflating subspace |
%   |                 |               corresponding to the infinite       |
%   |                 |               eigenvalues, negative if unknown    |
%   |                 |               (default -1)                        |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RankTol   - tolerance used in numerical rank    |
%   |                 |               detection                           |
%   |                 |               (default  log(n)*eps)               |
%   |                 |   RelTol    - tolerance for the relative change   |
%   |                 |               (default 1.0e+02*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | stabdecopts     | structure, containing the optional parameters for |
%   |                 | the decomposition of the stable and unstable parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, with                           |
%   |                 |   AbsTol    - tolerance for the absolute change   |
%   |                 |               (default 0)                         |
%   |                 |   Dimension - dimension of the deflating subspace |
%   |                 |               corresponding to the anti-stable    |
%   |                 |               eigenvalues, negative if unknown    |
%   |                 |               (default -1)                        |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RankTol   - tolerance used in numerical rank    |
%   |                 |               detection                           |
%   |                 |               (default  log(n)*eps)               |
%   |                 |   RelTol    - tolerance for the relative change   |
%   |                 |               (default 1.0e+02*n*eps)             |
%   |-----------------|---------------------------------------------------|
%   | StabMethod      | character array, determining algorithm for the    |
%   |                 | partial stabilization                             |
%   |                 |  'abe'  - algebraic Bernoulli equation based      |
%   |                 |  'lyap' - Bass' algorithm (Lyapunov equation)     |
%   |                 | (default 'abe')                                   |
%   |-----------------|---------------------------------------------------|
%   | stabmethodopts  | structure, containing the optional parameters for |
%   |                 | the sign function based Lyapunov or Bernoulli     |
%   |                 | equation solver used for the stabilization, with  |
%   |                 |   AbsTol    - tolerance for the absolute error    |
%   |                 |               (default 0)                         |
%   |                 |   Info      - disable/enable display of verbose   |
%   |                 |               status information during iteration |
%   |                 |               (default 0)                         |
%   |                 |   MaxIter   - maximum number of iteration steps   |
%   |                 |               (default 100)                       |
%   |                 |   RelTol    - tolerance for the relative error    |
%   |                 |               (default 1.0e+01*n*eps)             |
%   |-----------------|---------------------------------------------------|
%
% OUTPUT:
%   K    - stabilizing feedback matrix of dimensions m x n
%   info - structure, containing the following information about the
%          generalized partial stabilization method
%   |-----------------|---------------------------------------------------|
%   |      ENTRY      |                     MEANING                       |
%   |-----------------|---------------------------------------------------|
%   | infoINFDISK     | structure, containing information about the disk  |
%   |                 | function method used for the separation of the    |
%   |                 | infinite part, with                               |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABDISK    | structure, containing information about the disk  |
%   |                 | function method used for the separation of the    |
%   |                 | unstable part, with                               |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABDISK2   | structure, containing information about the disk  |
%   |                 | function method used for a second separation of   |
%   |                 | the unstable part if necessary, with              |
%   |                 |   AbsErr         - absolute change of iteration   |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative change of iteration   |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABMETH    | structure, containing information about the sign  |
%   |                 | function based solver used for the stabilization, |
%   |                 | with                                              |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | infoSTABMETH2   | structure, containing information about the sign  |
%   |                 | function based solver used for a second           |
%   |                 | stabilization if necessary, with                  |
%   |                 |   AbsErr         - absolute error of iteration    |
%   |                 |                    matrix                         |
%   |                 |   IterationSteps - number of iteration steps      |
%   |                 |   RelErr         - relative error of iteration    |
%   |                 |                    matrix                         |
%   |-----------------|---------------------------------------------------|
%   | Method          | character array, shortcut of the used             |
%   |                 | stabilization algorithm, with 'abe' for the       |
%   |                 | algebraic Bernoulli equation and 'lyap' for Bass' |
%   |                 | algorithm                                         |
%   |-----------------|---------------------------------------------------|
%   | Ninf            | Number of identified infinite eigenvalues         |
%   |-----------------|---------------------------------------------------|
%   | Ns              | Number of identified stable eigenvalues           |
%   |-----------------|---------------------------------------------------|
%   | Nu              | Number of identified anti-stable eigenvalues      |
%   |-----------------|---------------------------------------------------|
%
%
% REFERENCE:
%   P. BENNER. Partial stabilization of of descriptor systems using
%   spectral projectors. In P. Van Dooren, P. S. Bhattacharyya, H. R. Chan,
%   V. Olshevsky, and A. Routray, "Numerical Linear Algebra in Signals,
%   Systems and Control," chapter 3, pages 55-76. Springer Netherlands,
%   Dordrecht, 2011.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(3, 4);

if (nargin < 4) || isempty(opts)
    opts = struct();
end

% Check input matrices.
n = size(A, 1);
m = size(B, 2);

assert(isequal(size(A), [n n]), ...
    'MORLAB:data', ...
    'The matrix A has to be square!');

assert(size(B, 1) == n, ...
    'MORLAB:data', ...
    'The matrix B must have the same number of rows as A!');

assert(isequal(size(E), [n n]), ...
    'MORLAB:data', ...
    'The matrix E must have the same dimensions as A!');

% Check and assign optional parameters.
assert(isa(opts, 'struct'), ...
    'MORLAB:data', ...
    'The parameter opts has to be a struct!');

if ml_field_set_to_value(opts, 'Beta')
    ml_assert_nonnegscalar(opts.Beta, 'opts.Beta');
else
    opts.Beta = 0.1;
end

if ml_field_set_to_value(opts, 'DecompTol')
    ml_assert_nonnegscalar(opts.DecompTol, 'opts.DecompTol');
else
    opts.DecompTol = log(n) * eps;
end

if ml_field_set_to_value(opts, 'infdecopts')
    assert(isa(opts.infdecopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.infdecopts has to be a struct!');
else
    opts.infdecopts = struct();
end

if ml_field_set_to_value(opts, 'stabdecopts')
    assert(isa(opts.stabdecopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabdecopts has to be a struct!');
else
    opts.stabdecopts = struct();
end

if ml_field_set_to_value(opts, 'stabmethodopts')
    assert(isa(opts.stabmethodopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabmethodopts has to be a struct!');
else
    opts.stabmethodopts = struct();
end

if ml_field_set_to_value(opts, 'StabMethod')
    opts.StabMethod = lower(opts.StabMethod);
    
    assert(strcmp(opts.StabMethod, 'abe') ...
        || strcmp(opts.StabMethod, 'lyap'), ...
        'MORLAB:data', ...
        'The given stabilization method is not implemented!');
else
    opts.StabMethod = 'abe';
end

% Additional check of dimensions for avoiding unnecessary calls.
if ml_field_set_to_value(opts.infdecopts, 'Dimension')
    ml_assert_integer(opts.infdecopts.Dimension, ...
        'opts.infdecopts.Dimension');
else
    opts.infdecopts.Dimension = -1;
end

if ml_field_set_to_value(opts.stabdecopts, 'Dimension')
    ml_assert_integer(opts.stabdecopts.Dimension, ...
        'opts.stabdecopts.Dimension');
else
    opts.stabdecopts.Dimension = -1;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION: INFINITE PART.                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition into finite and infinite parts.
if opts.infdecopts.Dimension == 0
    ninf        = 0;
    infoINFDISK = [];
else
    svdE  = svd(E);
    alpha = svdE(sum(svdE > svdE(1) * opts.DecompTol)) / ...
        (1.01 * norm(sys.A, 'fro'));
    
    if isempty(alpha)
        alpha = 1.0;
    end
    
    [Aspace, ~, infoINFDISK] = ml_disk(E, alpha * A, opts.infdecopts);
    [Q, Z, ninf]             = ml_getqz(A, E, Aspace, opts.infdecopts);
end

if (ninf > 0) && (ninf < n)
    A = Q(: , ninf+1:end)' * A * Z(: , ninf+1:end);
    E = Q(: , ninf+1:end)' * E * Z(: , ninf+1:end);
    B = Q(: , ninf+1:end)' * B;
elseif ninf == n
    [A, B, E] = deal([]);
    Z         = speye(n);
else
    Z = speye(n);
end

n = n - ninf;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION: ANTI-STABLE PART.                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition into finite stable and unstable parts.
if opts.stabdecopts.Dimension >= 0
    opts.stabdecopts.Dimension = n - opts.stabdecopts.Dimension;
end

if (opts.stabdecopts.Dimension == 0) || (n == 0)
    ns           = 0;
    infoSTABDISK = [];
else
    [Aspace, ~, infoSTABDISK] = ml_disk(A + E, A - E, opts.stabdecopts);
    [U, V, ns]                = ml_getqz(A, E, Aspace, opts.stabdecopts);
end

if (ns > 0) && (ns < n)
    A = U(: , ns+1:end)' * A * V(: , ns+1:end);
    E = U(: , ns+1:end)' * E * V(: , ns+1:end);
    B = U(: , ns+1:end)' * B;
elseif ns == n
    [A, B, E] = deal([]);
    V         = speye(n);
else
    V = speye(n);
end

n = n - ns;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION (I).                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch opts.StabMethod
    case 'lyap' % Bass' Algorithm
        [X, infoSTABMETH] = ml_lyap_sgn(-(A + opts.Beta * E)', ...
            2 * (B * B'), E', opts.stabmethodopts);
        K1                = (E \ B)' * pinv(X);
        
    case 'abe' % Algebraic Bernoulli stabilization
        [X, infoSTABMETH] = ml_abe_sgn(A, B * B', E, opts.stabmethodopts);
        K1                = (B' * X) * E;
end

% Test of stability.
A                          = A - B * K1;
opts.stabdecopts.Dimension = n - sum(real(eig(A, E)) > 0);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION (II).                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if opts.stabdecopts.Dimension < n
    [Aspace, ~, infoSTABDISK2] = ml_disk(A + E, A - E, opts.stabdecopts);
    [U2, V2, ns2]              = ml_getqz(A, E, Aspace, opts.stabdecopts);
    
    if (ns2 > 0) && (ns2 < n)
        A = U2(: , ns2+1:end)' * A * V2(: , ns2+1:end);
        E = U2(: , ns2+1:end)' * E * V2(: , ns2+1:end);
        B = U2(: , ns2+1:end)' * B;
    elseif (ns2 == 0) || (ns2 == n)
        error('MORLAB:gpartstab', ...
            'Partial stabilization of the system failed!');
    end
    
    switch opts.StabMethod
        case 'lyap' % Bass' Algorithm
            opts.Beta          = 2 * norm(A, 'fro');
            [X, infoSTABMETH2] = ml_lyap_sgn(-(A + opts.Beta * E)', ...
                2 * (B * B'), E', opts.stabmethodopts);
            K2                 = (E \ B)' * pinv(X);
        case 'abe' % Algebraic Bernoulli stabilization
            [X, infoSTABMETH2] = ml_abe_sgn(A, B * B', E, ...
                opts.stabmethodopts);
            K2                 = (B' * X) * E;
    end
    
    if sum(real(eig(A - B * K2, E)) > 0)
        error('MORLAB:gpartstab', ...
            'Partial stabilization of the system failed!');
    end
    
    K = [zeros(m, ninf), [zeros(m, ns), K1 + ...
        [zeros(m, ns2), K2] * V2'] * V'] * Z';
else
    K = [zeros(m, ninf), [zeros(m, ns), K1] * V'] * Z';
    
    [infoSTABDISK2, infoSTABMETH2] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN INFORMATION.                                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Collection of all information.
info = struct(...
    'infoINFDISK'  , infoINFDISK, ...
    'infoSTABDISK' , infoSTABDISK, ...
    'infoSTABDISK2', infoSTABDISK2, ...
    'infoSTABMETH' , infoSTABMETH, ...
    'infoSTABMETH2', infoSTABMETH2, ...
    'Method'       , opts.StabMethod, ...
    'Ninf'         , ninf, ...
    'Ns'           , ns, ...
    'Nu'           , n);
